// Default gauge preferences
const GAUGE_WINDOW_SIZE = 400
const MIN_GAUGE_SIZE = 4;

// Default label preferences
const MIN_MAIN_LABEL_SIZE = 9;
const MAX_MAIN_LABEL_SIZE = 15;
const MAIN_LABEL_SIZE_OFFSET = MAX_MAIN_LABEL_SIZE - MIN_MAIN_LABEL_SIZE;
const MIN_SECONDARY_LABEL_SIZE = 11;
const MAX_SECONDARY_LABEL_SIZE = 17;
const SECONDARY_LABEL_SIZE_OFFSET = MAX_SECONDARY_LABEL_SIZE - MIN_SECONDARY_LABEL_SIZE;

/************************************************************************
	Gauge Object
*************************************************************************/	

function Gauge(id, prefs)
{
	// Initialize gauge
	this.Initialize = function()
	{
		if(debug) print("Adding gauge #" + id + "...");
	
		this.id = id;
		
		// Create bars array to hold the gauge's bars
		this.bars = new Array();
		
		// Assign default preferences
		this.title = "";
		this.source = "CPU";
                  this.updateFrequency = "-1";
		this.normalLevelColor = "#00FF00";
		this.warningLevelColor = "#FFFF00";
		this.criticalLevelColor = "#FF0000";
		this.size = 10;
		this.style = "Wave";
		this.labelPosition = "Right";
		this.barsDirection = 0;
		this.showLabel = 1;
		
		// If preferences were passed, assign them to the gauge object
		if(prefs != null && prefs != "default")
		{
			for(var ind in prefs)
			{
				this[ind] = prefs[ind];
			}
		}

		// Create gauge's window
		this.window = new Window("Window" + this.id);
		this.window.title = "Neon Gauges";
		this.window.level = preferences.konfabulatorWindowLevel.value;
		this.window.opacity = preferences.konfabulatorWindowOpacity.value;
		this.window.locked = preferences.konfabulatorLockWindowPosition.value;
		this.window.height = this.window.width = GAUGE_WINDOW_SIZE;
		this.window.onContextMenu = "tpTimer.ticking = tpTimer.show = false; gauges['Gauge " + this.id + "'].CreateContextMenu(); if(tooltipExists && tpWindow.visible) HideTooltip(); tpTimer.show = true; dummyWindow.focus();";
		
		// Create gauge's background image
		this.background = new Image(this.window);
		this.background.hAlign = this.background.vAlign = "center";
		this.background.hOffset = this.background.vOffset = GAUGE_WINDOW_SIZE/2;
		this.background.opacity = 50;
		
		// Create gauge's overlay image
		this.overlay = new Image(this.window);
		this.overlay.hAlign = this.overlay.vAlign = "center";
		this.overlay.hOffset = this.overlay.vOffset = GAUGE_WINDOW_SIZE/2;
		this.overlay.opacity = 100;
		
		// Create gauge's timer
		this.timer = new Timer();
		this.timer.onTimerFired = "gauge = gauges['Gauge " + this.id + "']; gauge.Update(true); if(tooltipExists && tpWindow.visible && selectedGauge == gauge) UpdateTooltip(gauge);";
		
		// Create gauge's labels
		this.label1 = new Text(this.window);
		this.label1.shadow = labelShadow;
		
		this.label2 = new Text(this.window);
		this.label2.shadow = labelShadow
			
		this.SetStyle(this.style);
		this.SetSource(this.source, this.updateFrequency);
		this.Resize(this.size);
		this.UpdateLabel(this.labelPosition);
		this.AssignTooltipEvents();
		this.Update();
		
		this.window.visible = true;
                  this.respondToGainFocus = true;
	  
                  // Make sure all gauges are unfocused
                  dummyWindow.focus();
		
		if(debug) print("Gauge #" + id + " has been added!");
	}
	
	// Create gauge's context menu
	this.CreateContextMenu = function()
	{
		var items = new Array();
		
		var ind = 0;
		
                items[ind] = new MenuItem();
		items[ind].title = "Create New Gauge...";
		items[ind].onSelect = "AddGauge();";

		ind++;
                
                items[ind] = new MenuItem();
		items[ind].title = "-";
		
		ind++;
                
		items[ind] = new MenuItem();
		items[ind].title = "Customize This Gauge...";
		items[ind].onSelect = "gauges['Gauge " + this.id + "'].EditPreferences();";
                
                ind++
		
		// Are there any other gauges?
		if(gaugesNumber > 1)
		{
			items[ind] = new MenuItem();
			items[ind].title = "Remove This Gauge";
			items[ind].onSelect = "RemoveGauge(" + this.id + ");";
			
			ind++;
		}
                
                items[ind] = new MenuItem();
		items[ind].title = "-";
		
		ind++;
		
		items[ind] = new MenuItem();
		items[ind].title = "More About WidgetWorld";
		items[ind].onSelect = "openURL('www.WidgetWorld.com');";
		
                ind++;
		
		items[ind] = new MenuItem();
		items[ind].title = "Check For Widget Updates";
		items[ind].onSelect = "CheckVersion(1);";
		
		this.window.contextMenuItems = items;
	}
	
	// Set gauge's style
	this.SetStyle = function(style)
	{
		try
		{
			this.timer.ticking = false;

			this.style = style;
			this.styleObj = GetStyle(style, this);

			this.background.src = "Resources/Images/Styles/" + this.style + "/background.png";
			this.overlay.src = "Resources/Images/Styles/" + this.style + "/" + (parseBool(preferences.useEnhancedGraphics.value) ? "shadowReflection" : "shadow") + ".png";
			
			this.ClearBars();
			
			// Create gauge's bars
			for(var i = 0; i < this.styleObj.barsNumber; i++)
			{
				this.bars[i] = new Image(this.window);
				this.bars[i].src = "Resources/Images/Styles/" + this.style + "/" + (i+1) + ".png";
				this.bars[i].hAlign = this.bars[i].vAlign = "center";
				this.bars[i].hOffset = this.bars[i].vOffset = GAUGE_WINDOW_SIZE/2;
			}
			
			// Make sure overlay image is placed in front of all other objects
			this.overlay.zOrder = this.bars[this.bars.length - 1].zOrder + 1;
			
			this.timer.ticking = true;
		}
		catch(e)
		{
			PrintError("SetStyle()", "Couldn't set gauge's style.", e);
		}
	}
	
	// Set gauge's source
	this.SetSource = function(source, updateFrequency)
	{	
		this.source = source;
                  this.updateFrequency = updateFrequency;
		this.sourceObj = GetSource(source, this);
		this.label1.data = this.title == "" ? this.sourceObj.GetCaption(this) : this.title;
		this.timer.interval = updateFrequency != "-1" ? updateFrequency : this.sourceObj.defaultUpdateFrequency;
		this.window.onMultiClick = "if(system.event.clickCount == 2){gauges['Gauge " + this.id  + "'].sourceObj.onDoubleClick(gauges['Gauge " + this.id  + "']); if(tpWindow.visible) HideTooltip();}";
	}
		
	// Update label's appearance and position
	this.UpdateLabel = function(position)
	{
		try
		{
			this.label1.font = preferences.labelFont.value;
			this.label2.font = preferences.labelFont.value;
		
			this.label1.visible = this.showLabel;
		
			var halfBkgWidth = this.background.width/2;
			var halfBkgHeight = this.background.height/2;
		
			this.label1.hOffset = this.label1.vOffset = this.label2.hOffset = GAUGE_WINDOW_SIZE/2;
		
			// If position isn't valid set it to 'Right'
			if(!position) position = "Right";

			switch(position)
			{
				case "Left":
					this.label1.hAlign = this.label2.hAlign = "right";
					
					this.label1.hOffset = this.label2.hOffset += this.size*this.styleObj.labelPosition.left.hOffset - halfBkgWidth - this.size/2;	
					this.label1.vOffset += this.size*this.styleObj.labelPosition.left.vOffset;
					
					this.label1.truncation = "end";
					this.label2.width = this.label1.width = this.label1.hOffset;
					
					this.label2.visible = this.showLabel;
				break;
				case "Right":
					this.label1.hAlign = this.label2.hAlign = "left";
					
					this.label1.hOffset = this.label2.hOffset += this.size*this.styleObj.labelPosition.right.hOffset + halfBkgWidth + this.size/2;	
					this.label1.vOffset += this.size*this.styleObj.labelPosition.right.vOffset;

					this.label1.truncation = "end";
					this.label2.width = this.label1.width = GAUGE_WINDOW_SIZE - this.label1.hOffset;
					
					this.label2.visible = this.showLabel;
				break;
				case "Top":
					this.label1.hAlign = "center";
					
					this.label1.hOffset = this.label2.hOffset += this.size*this.styleObj.labelPosition.top.hOffset;
					this.label1.vOffset += this.size*this.styleObj.labelPosition.top.vOffset - halfBkgHeight - this.size/2;
					
					this.label1.truncation = "center";
					this.label1.width = this.size*25;
		
					this.label2.visible = false;
				break;
				case "Bottom":
					this.label1.hAlign = "center";
					
					this.label1.hOffset = this.label2.hOffset += this.size*this.styleObj.labelPosition.bottom.hOffset;
					this.label1.vOffset += this.size*this.styleObj.labelPosition.bottom.vOffset + halfBkgHeight + this.size/0.7;
		
					this.label1.truncation = "center";
					this.label1.width = this.size*25;
		
					this.label2.visible = false;
				break;
			}
			
			this.label2.vOffset = this.label1.vOffset + this.label1.height;
	
			this.labelPosition = position;
		}
		catch(e)
		{
			PrintError("UpdateLabel()", "Couldn't set position of gauge label.", e);
		}
	}
		
	// Resize gauge
	this.Resize = function(size)
	{
		size = parseInt(size);
		
		ProportionalResize(this.background, size/10);
		ProportionalResize(this.overlay, size/10);
		
		for(var ind in this.bars)
		{
			ProportionalResize(this.bars[ind], size/10);
		}

		this.label1.size = MIN_MAIN_LABEL_SIZE + MAIN_LABEL_SIZE_OFFSET*(size - MIN_GAUGE_SIZE)/(10 - MIN_GAUGE_SIZE);
		this.label2.size = MIN_SECONDARY_LABEL_SIZE + MAIN_LABEL_SIZE_OFFSET*(size - MIN_GAUGE_SIZE)/(10 - MIN_GAUGE_SIZE);
		
                  this.overlay.reload();
                
		this.size = size;
	}
	
	// Assign tooltip-related events to the gauge's overlay and window.
    // These events are responsible for showing or hiding the tooltip when the gauge is hovered or clicked
	this.AssignTooltipEvents = function()
	{
		var onMouseEnter = "";
		var onMouseExit = "";
		var onGainFocus = "";
		
		// Create the events
		switch(preferences.tooltipDisplay.value)
		{
			case "Disabled":
				onMouseEnter = "";
				onMouseExit = "";
				onGainFocus = "";
			break;
			case "On Mouse Click":
				var gauge = this;
				
				onMouseEnter = "";
				onMouseExit = "tpTimer.ticking = !tpTimer.show;";
				onGainFocus = function()
				{
					// Make sure gauge should respond to onGainFocus events
					if(gauge.respondToGainFocus)
					{
						selectedGauge = gauge; 
						tpTimer.ticking = false;
						
						// If the tooltip doesn't exists, create it and show the tooltip.
						// Otherwise, if it's invisible, show it, if it's visible, rearrange and move it near the selected gauge
						if(!tooltipExists)
						{
							CreateTooltip(); 
							tpTimer.ticking = true;
						}
						else
						{
							if(!tpWindow.visible) tpTimer.ticking = true; // If a gauge is clicked:
							else if(gauge != tooltipGauge) ShowTooltip(false, true); // If a gauge is clicked while the tooltip has been shown:
						}
					}
				}
			break;
			default:
				var gauge = this;
			
				onMouseEnter = function()
				{
					// If the user quickly moves the mouse pointer from another gauge to this one, rearrange and move it near this gauge.
					// If the tooltip doesn't exists, show it near this gauge
					if(tooltipExists && tpWindow.visible && selectedGauge != gauge)
					{
						tpTimer.ticking = tpPositionTimer.ticking = false;
					
						selectedGauge = gauge;
						ShowTooltip(false, true);
					}
					else
					{
						selectedGauge = gauge; 
						tpTimer.ticking = tpTimer.show;
					}
				}
				onMouseExit = "if(selectedGauge == gauges['Gauge " + this.id + "']) tpTimer.ticking = !tpTimer.show;";
				onGainFocus = "";
			break;
		}
		
		// Assign the events to the objects
		this.overlay.onMouseEnter = this.label1.onMouseEnter = this.label2.onMouseEnter = onMouseEnter;
		this.overlay.onMouseExit = this.label1.onMouseExit = this.label2.onMouseExit = onMouseExit;
		this.window.onGainFocus = onGainFocus;
	}
		
	// Update gauge's display
	this.Update = function(animate)
	{
		try
		{
			// Get fata from source
			var data = this.sourceObj.GetData(this);

			// Make sure the data is valid
			if(data != "N/A" && parseInt(data).toString() != "NaN")
			{
				var color = this.normalLevelColor;
				
				switch(this.sourceObj.GetLevel(data))
				{
					case "normal": color = this.normalLevelColor; break;
					case "warning": color = this.warningLevelColor; break;
					case "critical": color = this.criticalLevelColor; break;
				}

				// Apply new color to gauge components
				this.label1.color = this.label2.color = this.background.colorize = color;
				
				for(var i = 0; i < this.bars.length; i++)
				{
					this.bars[i].colorize = color;
				
					var ind = i;
					if(this.barsDirection == 1)ind = this.bars.length - ind - 1;

					this.bars[i].opacity = 255*Math.min(1, Math.max(data/100*this.bars.length - ind, 0));
				}
				
				// Fade in the gauge in case it's was faded out
				
				if(this.window.opacity < preferences.konfabulatorWindowOpacity.value)
				{
					if(this.window.visible) animator.start(new FadeAnimation(this.window, preferences.konfabulatorWindowOpacity.value, 500, animator.kEaseOut));
					else this.window.opacity = preferences.konfabulatorWindowOpacity.value;
				}
			}
			else
			{
				// If source data is invalid, use low levels color and hide bars
				this.label1.color = this.label2.color = this.background.colorize = this.normalLevelColor;
				
				for(var i = 0; i < this.bars.length; i++)
				{
					this.bars[i].colorize = this.normalLevelColor;
					this.bars[i].opacity = 0;
				}
				
				// Fade out the gauge
				if(this.window.opacity > preferences.konfabulatorWindowOpacity.value/2)
				{
					// Don't use animations if the Widget is loading
					if(animate && this.window.visible) animator.start(new FadeAnimation(this.window, preferences.konfabulatorWindowOpacity.value/2, 500, animator.kEaseOut));
					else this.window.opacity = preferences.konfabulatorWindowOpacity.value/2;
				}
			}
			
			// Set label data based on its position
			if(this.labelPosition == "Left" | this.labelPosition == "Right")
			{
				this.label2.data = data;
				if(data != "N/A") this.label2.data += "%";
			}
			else
			{
				this.label1.data = (this.title == "" ? this.sourceObj.GetCaption(this) : this.title) + " " + data;
				if(data != "N/A") this.label1.data += "%";
			}
		}
		catch(e)
		{
			PrintError("Update()", "Couldn't update gauge.", e);
		}
	}
	
	// Edit gauge preferences
	this.EditPreferences = function()
	{
		// Get gauge preferences
		var prefs = GetGaugePreferences(this);
		
		// Apply preferences if they were changed
		if(prefs != null)
		{	
                           // Make sure tooltip doesn't show up is "On Mouse Click" mode is set
                           this.respondToGainFocus = false;
			this.window.visible = !(prefs.style != this.style | prefs.size != this.size); 

			this.title = prefs.title;
			this.barsDirection = prefs.barsDirection;
			this.showLabel = prefs.showLabel;
			this.normalLevelColor = prefs.normalLevelColor;
			this.warningLevelColor = prefs.warningLevelColor;
			this.criticalLevelColor = prefs.criticalLevelColor;

			this.disk = undefined;
		
			if(prefs.style != this.style) this.SetStyle(prefs.style);
			this.SetSource(prefs.source, prefs.updateFrequency);
			this.Resize(prefs.size);
			this.UpdateLabel(prefs.labelPosition);
			this.Update(true);

			this.window.visible = true;
                                
                            // Allow the tooltip to be shown
                            this.respondToGainFocus = true;
		}
                                              
                // Make sure all gauges are unfocused
                dummyWindow.focus();
	}
	
	// Remove gauge
	this.Remove = function()
	{
		this.window.visible = false;
	
		this.background.removeFromSuperview();
		this.overlay.removeFromSuperview();
		this.label1.removeFromSuperview();
		this.label2.removeFromSuperview();
		this.ClearBars();
	
		this.window = this.background = this.overlay = this.label1 = this.label2 = this.bars = this.timer = null;
	}
	
	// Clear gauge's bars
	this.ClearBars = function()
	{
		for(var i = this.bars.length - 1; i >= 0; i--)
		{
			this.bars[i].removeFromSuperview();
			this.bars[i] = null;
			this.bars.splice(i, 1);
		}
	}
	
	this.Initialize();
}