﻿/* 
	Neon Gauges 1.1
	Let your system info glow!
	
	© 2007 WidgetWorld.
*/

// Include additional files
include("Styles.js");
include("Sources.js");
include("Gauge.js");
include("Tooltip.js");

// Create global arrays to hold the gauges and styles
var gauges = new Array();			  
			  
var gaugesNumber = 0;
var debug = 1;

// Version Cheking Pieces
VERSION_NUMBER = '1.11';
VERSION_SHORTNAME = 'neongauges';
VERSION_URL = "http://versions.widgetworld.com/versions/" + VERSION_SHORTNAME + ".xml";			
VERSION_WIDGETNAME = "Neon Gauges";
			 
/************************************************************************
	Widget Management
*************************************************************************/	

Initialize();

// Initialize Widget
function Initialize()
{	
	if(debug) PrintFunction("Initialize");
	
	// Create global label shadow object
	labelShadow = new Shadow()
	labelShadow.color = "#000000";
	labelShadow.vOffset = -1;
	labelShadow.opacity = 90;
	
         // Create a different tooltipDisplay preference for each plaform because "On Mouse Over" mode doesn't work well on the Mac 
	if(system.platform == "macintosh")
	{
		preferences.tooltipDisplay.option = new Array("Disabled", "On Mouse Click");
		preferences.tooltipDisplay.optionValue = new Array("Disabled", "On Mouse Click");
	}
	if(system.platform == "windows")
	{
		preferences.tooltipDisplay.option = new Array("Disabled", "On Mouse Over", "On Mouse Click");
		preferences.tooltipDisplay.optionValue = new Array("Disabled", "On Mouse Over", "On Mouse Click");
	}

	// Check if the Widget has been loaded yet
	if(preferences.firstLoad.value.split("|")[1] != "1")
	{
		// The Widget is loaded for the first time.
		if(debug) print("\nFirst load. Setting default values...");
		
		if(system.platform == "macintosh") preferences.tooltipDisplay.value = "On Mouse Click";
		if(system.platform == "windows") preferences.tooltipDisplay.value = "On Mouse Over";
		
		// Set the firstLoad preference to mark that the current version of the Widget has already been loaded for the first time
		preferences.firstLoad.value = "1.0.1|1";
		savePreferences();
	}
	
	// The Mac version doesn't allow moving a window which hasn't been visible first so the tooltip must be shown and hide as a workaround. 
	tpWindow.visible = !(tpWindow.visible = true);
        
	// Load gauges from XML
	LoadGauges();
	
	if(parseBool(preferences.autoCheckVersion.value)) CheckVersion();
}

// Unload Widget
function Unload()
{
	if(debug) PrintFunction("Unload");
		
	// Save gauges before unloading
	SaveGauges();
}

// Apply preferences if the user chooses to save them 
function ApplyPreferences()
{
	if(debug) PrintFunction("ApplyPreferences");
		
	// Make sure tooltip's window is locked and 'topmost'
	if(tooltipExists) 
	{
		tpWindow.level = "topmost";
		tpWindow.locked = true;
		tpWindow.opacity = 0;
		
		preferences.tooltipDisplay.value != "Disabled" ? UpdateTooltipSkin() : DestroyTooltip();
	}
		
	for(var ind in gauges)
	{
		try
		{
			var gauge = gauges[ind];
			
			gauge.UpdateLabel(gauge.labelPosition);
			gauge.AssignTooltipEvents();
			
			gauge.overlay.src = "Resources/Images/Styles/" + gauge.style + "/" + (parseBool(preferences.useEnhancedGraphics.value) ? "shadowReflection" : "shadow") + ".png";
			ProportionalResize(gauge.overlay, gauge.size/10);
			
			gauge.Update();
		}
		catch(e)
		{
			PrintError("ApplyPreferences()", "Couldn't apply preferences to gauge #" + ind + ".", e);
		}
	}
	
	tpTimer.updateInterval();
        
	// Make sure all gauges are unfocused
	dummyWindow.focus();
}

// Check for version updates
function CheckVersion(respondRegardless)
{		
	if (debug) print("\n--------------------------\nFunction: checkVersion\n");

	var versionRequest = new XMLHttpRequest();	

	versionRequest.onreadystatechange = function ()
	{	
		if( debug > 2 ) print("\n XML State has changed. Status is: " + this.readyState);
		
		if(this.readyState == 4) // complete
		{
			try
			{
				var doc = this.responseXML;
			
				if(this.status == 200 && doc != null)
				{	
					// Okay, looks like we have a version XML file. 
					var funds = doc.evaluate(VERSION_SHORTNAME + "/version_tag");
					
					// grab the first (and should be only) item and get the version 
					// and the downInitialize URL
					var node = funds.item(0);
		
					var tmpVersion = node.evaluate("string(version)");
					var downInitializeURL = node.evaluate("string(url)");
							
					if ( debug > 2 ) print ("tmpVersion = " + tmpVersion + "; VERSION_NUMBER = " + VERSION_NUMBER);		
							
					// If things look good, compare the numbers and show them their prizes.
					if ((tmpVersion != null) && (downInitializeURL != null)){
						if (Number(VERSION_NUMBER) < Number(tmpVersion)){
							var response = alert ( "There is a newer version of the " + VERSION_WIDGETNAME + " widget available.\n\nDo you want to download it?", "Yes", "No");
							if (response == 1) openURL(downInitializeURL);
						}else if (respondRegardless == 1){
							alert("You are running the most recent version of " + VERSION_WIDGETNAME + ".");
						}	
					}
					else{
						alert ( "Error: Version information could not be retrieved from server." );
					}
				}	
				else
				{
					if (respondRegardless == 1)
					alert("Error: Version information could not be retrieved from server.");
				} // end of if this.status
			}
			catch(e)
			{
				 if (respondRegardless == 1)
					alert("Error: Version information could not be retrieved from server.");
			}
			
		} // end of if this.readyState		

	} // end of onreadystate function 

	// send async request			
	versionRequest.open( "GET", VERSION_URL, true);	
	versionRequest.send();
}

/************************************************************************
	Gauge Management
*************************************************************************/	

// Adds new gauges and applies their preferences if they are loaded from memory
function AddGauge(prefs, id)
{
	if(debug) PrintFunction("AddGauge");
	
	// If no preferences are passed, the gauge isn't loaded from memory so the user is prompted to set its preferences.
	if(prefs == null)
	{
		prefs = GetGaugePreferences();
		
		// If the user clicked the "Cancel" button so no gauge should be added and the function must stop
		if(prefs == null) return;
	}
	
	// If no ID is passed, a new ID must be given to the gauge
	if(!id) id = new Date().getTime();
	
	// Create the new gauge, add it to the gauges collection and increse the gagues number
	gauges["Gauge " + id] = new Gauge(id, prefs);
	gaugesNumber++;
}

// Removes a gauge according to its ID
function RemoveGauge(id)
{
	if(debug) PrintFunction("RemoveGauge");

	// Do we have a reference to a gauge?
	if(gauges["Gauge " + id] instanceof Gauge)
	{
		// Does the user really want to remove the gauge?
		if(alert("Are you sure you want to remove this gauge?", "Yes", "No") == 1)
		{
			if(debug) print("Removing gauge #" + id + "...");
			// Tell the guage to remove its content
			gauges["Gauge " + id].Remove();
			
			// Delete gauge from gauges collection
			delete gauges["Gauge " + id];
			
			// Decrease gauges number
			gaugesNumber--;
			
			if(debug) print("Gauge #" + id + " has been removed!");
		}
	}
	else
	{
		PrintError("RemoveGauge()", "Wrong gauge ID: " + id);
	}
}

// Builds and shows the "Gauge Preferences" dialog for the gauge whose ID is passed or for a new gauge
function GetGaugePreferences(gauge)
{
	if(debug) PrintFunction("GetGaugePreferences");

	var items = new Array();
		
	var ind = 0;
	
	items[ind] = new FormField();
	items[ind].type = "text";
	items[ind].title = "Title:";
	items[ind].defaultValue = "";
	items[ind].description = "Gauge's title. Leave this field empty for default title.";
	if(gauge) items[ind].value = gauge.title;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "popup";
	items[ind].title = "Source:";
	items[ind].option = GetSourcesNames();
	items[ind].defaultValue = "CPU";
	if(gauge) items[ind].value = gauge.source;
	
	ind++;
        
        items[ind] = new FormField();
        items[ind].type = "popup";
        items[ind].title = "Update Frequency:";
        items[ind].option = ["Default", "-", "1 second", "2 seconds", "3 seconds", "4 seconds", "5 seconds", "6 seconds", "7 seconds", "8 seconds", "9 seconds", "10 seconds", "15 seconds", "20 seconds", "30 seconds", "45 seconds", "1 minute", "2 minutes", "5 minutes"];
        items[ind].optionValue = [-1, "", 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, 20, 30, 45, 60, 120, 300]
        items[ind].defaultValue = -1;
        if(gauge) items[ind].value = gauge.updateFrequency;
        
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "popup";
	items[ind].title = "Style:";
	items[ind].option = GetStylesNames();
	items[ind].defaultValue = "Wave";
	if(gauge) items[ind].value = gauge.style;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "checkbox";
	items[ind].title = "Reverse Bars Direction";
	items[ind].description = "Reverse the direction from which the bars are being drawn on the gauge.";
	items[ind].defaultValue = 0;
	if(gauge) items[ind].value = gauge.barsDirection;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "checkbox";
	items[ind].title = "Show Label";
	items[ind].defaultValue = 1;
	if(gauge) items[ind].value = gauge.showLabel;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "popup";
	items[ind].title = "Label Position:";
	items[ind].option = new Array("Left", "Right", "Top", "Bottom");
	items[ind].defaultValue = "Right";
	items[ind].description = "Show the label in this side of the gauge.";
	if(gauge) items[ind].value = gauge.labelPosition;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "color";
	items[ind].title = "Normal Level:";
	items[ind].defaultValue = "#00FF00";
	if(gauge)items[ind].value = gauge.normalLevelColor;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "color";
	items[ind].title = "Warning Level:";
	items[ind].defaultValue = "#FFFF00";
	if(gauge) items[ind].value = gauge.warningLevelColor;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "color";
	items[ind].title = "Critical Level:";
	items[ind].defaultValue = "#FF0000";
	items[ind].description = "These colors will be used to colorize the gauge when it reaches normal, warning or critical levels.";
	if(gauge) items[ind].value = gauge.criticalLevelColor;
	
	ind++;
	
	items[ind] = new FormField();
	items[ind].type = "slider";
	items[ind].title = "Size:";
	items[ind].minLength = MIN_GAUGE_SIZE;
	items[ind].maxLength = 10;
	items[ind].ticks = 11 - MIN_GAUGE_SIZE;
	items[ind].tickLabel = new Array("Small", "Large");
	items[ind].defaultValue = 10;
	if(gauge) items[ind].value = gauge.size;
	
	var results = form(items, "Gauge Customization", gauge ? "OK" : "Create Gauge");
	
	// Return the results from the preferences dialog
	return results ? {title: results[0], source: results[1], updateFrequency: results[2], style: results[3], barsDirection: results[4], showLabel: results[5], labelPosition: results[6], normalLevelColor: results[7], warningLevelColor: results[8], criticalLevelColor: results[9], size: results[10]} : null;
}

// Loads gauges from memory
function LoadGauges()
{
	if(debug) PrintFunction("LoadGauges");

	var loadDefaultGauge = true;
	
	// Are there any gauges saves in memeory?
	if(preferences.gauges.value != null && preferences.gauges.value != "")
	{
		if(debug) print("Preparing to parse XML file...");
		
		var doc = XMLDOM.parse(preferences.gauges.value)
		
		// Do we have a valid XML document?
		if(doc != null)
		{
			if(debug) print("XML has been parsed successfully! Starting to load data from XML...");
		
			try
			{
				// Load the <gauge> nodes
				var gaugeNodes = doc.evaluate("Gauges/Gauge");
			
				var gaugeNodesNumber = gaugeNodes.length;
				
				// Are there any gauges specified in the XML?
				if(gaugeNodesNumber != null && gaugeNodesNumber > 0)
				{
					// No need to load default gauge
					loadDefaultGauge = false;
					
					if(debug >= 2) print("Gauges found! Starting to add gauges...");
					
					// Add each gauge and apply its preferences
					for(var id = 0; id < gaugeNodesNumber; id++)
					{
						if(debug >= 2) print("Loading gauge #" + id + "...");
						
						try
						{
							var gaugeAttributes = gaugeNodes.item(id).attributes;
						
							var prefs = new Object();
							
							for(var i = 0; i < gaugeAttributes.length; i++)
							{
								prefs[gaugeAttributes.item(i).name] = gaugeAttributes.item(i).value;
							}

							AddGauge(prefs, prefs["id"]);
						}
						catch(e)
						{
							PrintError("LoadGauges()", "Couldn't load gauge #" + id + ".", e);
						}
					}
				}
				else PrintError("LoadGauges()", "No gauges are specified in the XML.");
			}
			catch(e) 
			{
				PrintError("LoadGauges()", "Couldn't load gauges from XML.", e);
			}
		}
		else PrintError("LoadGauges()", "Invalid XML document was detected.");
	}

	// Load the default gauge if needed
	if(loadDefaultGauge | gaugesNumber == 0)
	{
		if(debug) print("\nLoading default gauge...");
		AddGauge("default");	
	}
	else
	{
		if(debug) print("\nFinished loading gauges!");
	}
}

// Saves gauges to memory
function SaveGauges()
{
	if(debug) PrintFunction("SaveGauges");
	
	// In case of a critical error, the Widget might not load any gauges so there's no need to save anything
	if(gaugesNumber > 0)
	{
		// Prepare XML document
		var doc = XMLDOM.createDocument();

		// Create <Gauges> node
		var gaugesNode = doc.createElement("Gauges");
		
		// Save each gauge's preferences
		for(var id in gauges)
		{	
			// Make sure we have a gauge object
			if(gauges[id] instanceof Gauge)
			{
				if(debug >= 2) print("Saving gauge #" + id + "...");
					
				try
				{
					// Prepare <Gauge> node
					var gaugeNode = doc.createElement("Gauge");
					var gauge = gauges[id];
					
					gaugeNode.setAttribute("id", gauge.id);
					gaugeNode.setAttribute("title", gauge.title);
					gaugeNode.setAttribute("source", gauge.source);
                                            gaugeNode.setAttribute("updateFrequency", gauge.updateFrequency);
					gaugeNode.setAttribute("normalLevelColor", gauge.normalLevelColor);
					gaugeNode.setAttribute("warningLevelColor", gauge.warningLevelColor);
					gaugeNode.setAttribute("criticalLevelColor", gauge.criticalLevelColor);
					gaugeNode.setAttribute("size", gauge.size);
					gaugeNode.setAttribute("style", gauge.style);
					gaugeNode.setAttribute("labelPosition", gauge.labelPosition);
					gaugeNode.setAttribute("barsDirection", gauge.barsDirection);
					gaugeNode.setAttribute("showLabel", gauge.showLabel);
					
					gaugesNode.appendChild(gaugeNode);
				}
				catch(e)
				{
					PrintError("SaveGauges()", "Couldn't save gauge #" + id + ".", e);
				}
			}
		}
		
		doc.appendChild(gaugesNode);

		// Save to the gauges preference
		preferences.gauges.value = doc.toXML();
                
		savePreferences();
		
		if(debug) print("\nFinished saving gauges!");
	}
}

/************************************************************************
	Helper Functions
*************************************************************************/	
 
// Calculates a percentage value
function GetPercent(val, total)
{
	return parseInt(100*val/total);
}

// Resize an image by percent
function ProportionalResize(image, percent)
{
	image.width = image.srcWidth*percent;
	image.height = image.srcHeight*percent;
}

// Convert an integer value representing minutes to a corresponding hours:minutes string
function GetTimeAsString(minutes)
{
	if(minutes >= 0)
	{
		var hours = 0;
		
		hours = Math.floor(minutes/60);
		minutes = Math.floor(minutes%60);

		return (hours < 10 ? "0" + hours : hours) + ":" + (minutes < 10 ? "0" + minutes : minutes);
	}
	else return "Error";
}

// Returns true if the passed value can be converted to true or 1, otherwise returns false
function parseBool(val)
{
	return (val == "true" | val == "1")
}

/************************************************************************
	Debug
*************************************************************************/	

// Print the currently running function's name to debug window
function PrintFunction(func)
{
	print("\n-----------------------------------\nFunction:  " + func + "()");
}

// Print an error message to debug window according to the passed function name, message and error object
function PrintError(func, message, e)
{
	if(e instanceof Error)
	{
		print("ERROR: " + func + " >> " + message + " The error message is: " + e.message + " (File: " + e.fileName + ", Line:" + e.lineNumber  + ")");
	}
	else if(e instanceof DOMException)
	{
		print("ERROR: " + func + " >> " + message + " Error code: " + e.code);
	}
	else
	{
		print("ERROR: " + func + " >> " + message);
	}
}