// Create global array to hold sources
var sources = new Array();

/************************************************************************
	Sources Management
*************************************************************************/	

// Adds sources to the sources collection
function RegisterSource(source, isDefault)
{
	sources.push(source);
	
	// Set default source
	if(isDefault) this.sources.defaultSource = source;
}

// Returns a source based on its name
function GetSource(sourceName, gauge)
{
	for(var i = 0; i < sources.length; i++)
	{
		if(sourceName.match("^" + sources[i].name)) return sources[i];
	}
	
	gauge.source = "CPU";
	return sources.defaultSource;
}

// Returns an array of available sources names
function GetSourcesNames()
{
	var sourcesNames = new Array();
	
	for(var i = 0; i < sources.length; i++)
	{
		sourcesNames = sourcesNames.concat(sources[i].GetSourcesNames());
	}
	
	return sourcesNames;
}

/************************************************************************
	Sources
*************************************************************************/	

function CPUSource()
{
	this.name = "CPU";
	this.defaultUpdateFrequency = 1;
	
	this.GetCaption = function()
	{
		return this.name + ":";
	}
	
	this.onDoubleClick = function()
	{
		if(system.platform == "windows") runCommandInBg("$SystemRoot/system32/taskmgr.exe", "runTaskManager");
		if(system.platform == "macintosh") filesystem.open("/Applications/Utilities/Activity Monitor.app");
	}
	
	this.GetSourcesNames = function()
	{
		return this.name;
	}
	
	this.GetData = function()
	{
		return system.cpu.activity;
	}
	
	this.GetLevel = function(data)
	{
		if(data >= 35)
		{
			if(data >= 70) return "critical";
			else return "warning";
		}
		else return "normal";
	}
	
	this.GetTooltipContent = function()
	{
		return {caption: this.name, rows:
			   [{item: "Processors:", value: system.cpu.numProcessors},
   			    {item: "Activity:", value: system.cpu.activity + "%"},
			    {item: "System tasks:", value: system.cpu.sys + "%"},
			    {item: "User tasks:", value: system.cpu.user + "%"},
			    {item: "Idle:", value: system.cpu.idle + "%"}]}
	}
}

RegisterSource(new CPUSource(), true);

function RAMSource()
{
	this.name = "RAM";
	this.defaultUpdateFrequency = 20;
	
	this.GetCaption = function()
	{
		return this.name + ":";
	}
	
	this.onDoubleClick = function()
	{
	}
	
	this.GetSourcesNames = function()
	{
		return this.name;
	}
	
	this.GetData = function()
	{
		return system.memory.load;
	}
	
	this.GetLevel = function(data)
	{
		if(data > 50)
		{
			if(data > 75) return "critical";
			else return "warning";
		}
		else return "normal";
	}
	
	this.GetTooltipContent = function()
	{
		return {caption: this.name, rows:
			   [{item: "Total RAM:", value: bytesToUIString(system.memory.totalPhysical)},
   			    {item: "Available RAM:", value: bytesToUIString(system.memory.availPhysical)},
			    {item: "Memory used:", value: system.memory.load + "%"}]}
	}
}

RegisterSource(new RAMSource());

function SwapSource()
{
	this.name = "Swap";
	this.defaultUpdateFrequency = 20;
	
	this.GetCaption = function()
	{
		return this.name + ":";
	}
	
	this.onDoubleClick = function()
	{
	}
	
	this.GetSourcesNames = function()
	{
		return this.name;
	}
	
	this.GetData = function()
	{
		var swapData = this.GetSwapData();
		
		if(swapData != "N/A") return GetPercent(swapData.totalVirtual - swapData.availVirtual, swapData.totalVirtual);
		else return "N/A";
	}
	
	this.GetLevel = function(data)
	{
		if(data > 50)
		{
			if(data > 75) return "critical";
			else return "warning";
		}
		else return "normal";
	}
	
	this.GetTooltipContent = function()
	{
		var swapData = this.GetSwapData();
		
		if(swapData != "N/A")
		{
			return {caption: this.name, rows:
				   [{item: "Total virtual:", value: bytesToUIString(swapData.totalVirtual)},
					{item: "Free virtual:", value: bytesToUIString(swapData.availVirtual)},
					{item: "Swap used:", value: GetPercent(swapData.totalVirtual - swapData.availVirtual, swapData.totalVirtual) + "%"}]}
		}
		else return {caption: this.name, rows:  [{item: "Not Available", value: ""}]};
	}
	
	this.GetSwapData = function()
	{
		var totalVirtual;
		var availVirtual;
		
		if (system.platform == "windows")
		{
			try
			{
				if(!this.wmi) this.wmi = COM.createObject("WbemScripting.SWbemLocator.1").ConnectServer(".", "root/cimv2");
				
				var swap = this.wmi.Get("Win32_PerfRawData_PerfOS_Memory=@").Properties_;
				totalVirtual = swap.Item("CommitLimit").Value;
				availVirtual = totalVirtual - swap.Item("CommittedBytes").Value;
			}
			catch(e)
			{
				PrintError("GetSwapData()", "Couldn't get swap data.", e);
				return "N/A";
			}
		}
		else
		{
			totalVirtual = system.memory.totalVirtual;
			availVirtual = system.memory.availVirtual;
		}
	
		return {totalVirtual: totalVirtual, availVirtual: availVirtual}
	}
}

RegisterSource(new SwapSource());

function WiFiSource()
{
	this.name = "WiFi";
	this.defaultUpdateFrequency = 8;
	
	this.GetCaption = function()
	{
		return this.name + ":";
	}
	
	this.onDoubleClick = function()
	{
	}
	
	this.GetSourcesNames = function()
	{
		return this.name;
	}
	
	this.GetData = function()
	{
		if(system.wireless.available && system.wireless.powered)
		{
			return Math.round(Math.min(70, system.wireless.signal) * 1.43);
		}
		else	return "N/A";
	}
	
	this.GetLevel = function(data)
	{
		if(data > 50)
		{
			if(data > 79) return "normal";
			else return "warning";
		}
		else return "critical";
	}
	
	this.GetTooltipContent = function()
	{
		if(system.wireless.available)
		{
			if(system.wireless.powered)
			{
				return {caption: this.name, rows:
					   [{item: "Network:", value: system.wireless.network},
					    {item: "Signal quality:", value: this.GetData() + "%"}]}
			}
			else return {caption: this.name, rows: [{item: "Offline", value: ""}]};
		}
		else return {caption: this.name, rows: [{item: "Not available", value: ""}]};
	}
}

RegisterSource(new WiFiSource());

function BatterySource()
{
	this.name = "Battery";
	this.defaultUpdateFrequency = 4;
	
	this.GetCaption = function()
	{
		return this.name + ":";
	}
	
	this.onDoubleClick = function()
	{
	}
	
	this.GetSourcesNames = function()
	{
		return this.name;
	}
	
	this.GetData = function()
	{
		var batteryData = this.GetBatteryData();
				
		return (batteryData != "Information not available" && batteryData != "No Battery") ? batteryData.capacity : "N/A";
	}
	
	this.GetLevel = function(data)
	{
		if(data > 25)
		{
			if(data > 75) return "normal";
			else return "warning";
		}
		else return "critical";
	}
	
	this.GetTooltipContent = function()
	{
		var batteryData = this.GetBatteryData();
		
		if(batteryData != "Information not available" && batteryData != "No Battery")
		{
			return {caption: this.name, rows:
			       [{item: "Batteries  #:", value: batteryData.batteryCount},
					{item: "Status:", value: batteryData.status},
					{item: "Current capacity:", value: batteryData.capacity + "%"},
					{item: batteryData.timeItem, value: batteryData.time}]}
		}
		else
		{
			return {caption: this.name, rows: [{item: batteryData, value: ""}]};
		}		
	}
	
	this.GetBatteryData = function()
	{
		// Continue only if at least one of the batteries is available and present in the system
		if((system.battery[0] && system.battery[0].isPresent) || (system.battery[1] && system.battery[1].isPresent))
		{
			var capacity = 0;
			var	maximumCapacity = 0;
			var time = 0;
			var timeItem;
			var status = false;
		
			try
			{
				// Get batteries information
				if(system.battery[0] && system.battery[0].isPresent)
				{
					capacity = system.battery[0].currentCapacity;
					maximumCapacity = system.battery[0].maximumCapacity;
					status = system.battery[0].isCharging;
					
					if(system.battery[0].powerSourceState == "AC Power")
					{
						if(system.battery[0].timeToFullCharge == -1) time = "Calculating";
						else time = system.battery[0].timeToFullCharge;
					}
					else
					{
						if(system.battery[0].timeToEmpty == -1) time = "Calculating";
						else time = system.battery[0].timeToEmpty;
					}
				}
				
				if(system.battery[1] && system.battery[1].isPresent)
				{
					capacity += system.battery[1].currentCapacity;
					maximumCapacity += system.battery[1].maximumCapacity;
					if(!status) status = system.battery[1].isCharging;
					
					if(time != "Calculating")
					{
						if(system.battery[1].powerSourceState == "AC Power")
						{
							if(system.battery[1].timeToFullCharge == -1) time = "Calculating";
							else time += system.battery[1].timeToFullCharge;
						}
						else
						{
							if(system.battery[1].timeToEmpty == -1) time = "Calculating";
							else time += system.battery[0].timeToEmpty;
						}
					}
				}
				
				if(time != "Calculating") time = GetTimeAsString(time);
				status = status  ? "Charging" : "Charged";
			
				timeItem = system.battery[0].powerSourceState == "AC Power" ? "Time to full" : "Time to empty";
				
				return {capacity: capacity/(maximumCapacity/100), batteryCount: system.batteryCount, status: status, timeItem: timeItem, time: time};
			}
			catch(e)
			{
				PrintError("GetBatteryData()", "Couldn't get battery data.", e);
				return "Information not available";
			}
		}
		else return "No Battery";
	}
}

RegisterSource(new BatterySource());

function DiskSource()
{
	this.name = "Disk";
	this.defaultUpdateFrequency = 5;
	
	this.GetCaption = function(gauge)
	{
		if(!gauge.disk) gauge.disk = gauge.source.match(/Disk ([^>]+)/)[1];
		return "HD " + gauge.disk;
	}
	
	this.onDoubleClick = function(gauge)
	{
		try
		{
			if(!gauge.disk) gauge.disk = gauge.source.match(/Disk ([^>]+)/)[1];
			filesystem.open(gauge.disk);
		}
		catch(e)
		{
			PrintError("onDoubleClick()", "Couldn't open drive " + gauge.source.match(/Disk ([^>]+)/)[1] + " .", e);
			alert("Couldn't open drive " + gauge.source.match(/Disk ([^>]+)/)[1] + " !\n\nError message: " + e.message);
		}
	}
	
	this.GetSourcesNames = function()
	{
		var volumes = filesystem.volumes;

		for(var ind in volumes)
		{
			volumes[ind] = "Disk " + volumes[ind].path;
		}
	
		return volumes;
	}
	
	this.GetData = function(gauge)
	{
		var diskIndex = this.GetDiskIndex(gauge);
		
		// Was the disk index found?
		if(diskIndex >= 0)
		{
			var disk = filesystem.volumes[diskIndex];
		
			var percent = GetPercent(disk.totalBytes - disk.freeBytes, disk.totalBytes);
			
			// Do we have a percentage value?
			if(percent.toString() == "NaN") percent = "N/A";
		
			return percent;
		}
		else return "N/A";
	}
	
	this.GetLevel = function(data)
	{
		if(data > 50)
		{
			if(data > 75) return "critical";
			else return "warning";
		}
		else return "normal";
	}
	
	this.GetTooltipContent = function(gauge)
	{
		var totalSize;
		var freeSpace;
		var spaceUsed;
		
		var diskIndex = this.GetDiskIndex(gauge);
		
		// Was the disk index found?
		if(diskIndex >= 0)
		{
			var disk = filesystem.volumes[diskIndex];
			
			// Make sure we can get valid data about this disk
			if(disk.totalBytes > 0)
			{
				totalSize = bytesToUIString(disk.totalBytes);
				freeSpace = bytesToUIString(disk.freeBytes);
				spaceUsed = GetPercent(disk.totalBytes - disk.freeBytes, disk.totalBytes);
				
				return {caption: gauge.source, rows:
					   [{item: "Total size:", value: totalSize},
					    {item: "Free space:", value: freeSpace},
					    {item: "Space used:", value: spaceUsed + "%"}]}
			}
			else return {caption: gauge.source, rows: [{item: "Information not available", value: ""}]};
		}
		else return {caption: gauge.source, rows: [{item: "Not available", value: ""}]};
	}
	
	this.GetDiskIndex = function(gauge)
	{
		var disk = gauge.source.match(/Disk ([^>]+)/)[1];
		
		// Search for disk index
		for(var ind in filesystem.volumes)
		{
			if(filesystem.volumes[ind].path == disk) return ind;
		}
		
		return -1;
	}
}

RegisterSource(new DiskSource());