// Create global arrays to hold the tooltip's text objects
var tpRowsItems = new Array();
var tpRowsValues = new Array();

var selectedGauge;
var selectedGaugeHoffset;
var selectedGaugeVoffset;

var tooltipExists = false;
var tooltipSide;
var tooltipGauge;

// Default tooltip settings
TOOLTIP_OPACITY = 240;
TOOLTIP_MOUSEOVER_DELAY = 0.5;
TOOLTIP_MOUSECLICK_DELAY = 0.1;

// Create tooltip timer which is responsible for showing or hiding the tooltip
var tpTimer = new Timer();
tpTimer.updateInterval = function(){this.interval = preferences.tooltipDisplay.value == "On Mouse Over" ? TOOLTIP_MOUSEOVER_DELAY : TOOLTIP_MOUSECLICK_DELAY;}
tpTimer.ticking = false;
tpTimer.onTimerFired = "tpTimer.ticking = false; ToggleShowHideTooltip();";
tpTimer.show = true;
tpTimer.updateInterval();

// Show or hide the tooltip once the tooltip timer is fired
function ToggleShowHideTooltip()
{
	if(tpTimer.show)
	{
		if(!tooltipExists) CreateTooltip();
		if(!tpWindow.visible) ShowTooltip(true);	
	}
	else HideTooltip();
}

// Create the tooltip
function CreateTooltip()
{
	if(debug) PrintFunction("CreateTooltip");

	tpWindow.level = "topmost";
	tpWindow.locked = true;
        
	// Create tooltip arrow
	tpArrow = new Image(tpWindow);

	// Create tooltip's background images
	tpTopLeft = new Image(tpWindow);
	tpTop = new Image(tpWindow);
	tpTopRight = new Image(tpWindow);
	tpLeft = new Image(tpWindow);
	tpCenter = new Image(tpWindow);
	tpRight = new Image(tpWindow);
	tpBottomLeft = new Image(tpWindow);
	tpBottom = new Image(tpWindow);
	tpBottomRight = new Image(tpWindow);
	
	// Make sure the arrow appears above the right section
	tpArrow.zOrder = tpRight.zOrder + 1;
	
	// Create tooltip separator
	tpSeparator = new Image(tpWindow);
	
	UpdateTooltipSkin();
	
	// Create tooltip's text shadow
	tpTextShadow = new Shadow();
	tpTextShadow.hOffset = 0;
	tpTextShadow.vOffset = -1;
	tpTextShadow.color = "#000000";

	// Create tooltip's caption
	tpCaption = new Text(tpWindow);
	tpCaption.color = "#FFFFFF";
	tpCaption.shadow = tpTextShadow;
	tpCaption.hOffset = iHoffset + 10;
	tpCaption.vOffset = 17;
	tpCaption.style = "bold";
	tpCaption.truncation = "end";
	
	// Create tooltip position timer which is responsible for focusing the tooltip next to the hovered gauge
	tpPositionTimer = new Timer();
	tpPositionTimer.interval = 1;
	tpPositionTimer.ticking = false;
	tpPositionTimer.onTimerFired = "if(tpWindow.visible){if(system.platform == 'windows') tpWindow.focus(); SetTooltipPosition(selectedGauge);}";
	
	// "On Mouse Over" feature implementation - works only in the Windows version
	if(system.platform == "windows")
	{
		// Assign events to window's objects to prevent tooltip from moving and hiding itself whie the mouse pointer hovers it
		for(var i = 0 ; i < tpWindow.root.subviews.length; i++)
		{
			tpWindow.root.subviews[i].onMouseEnter = "tpTimer.ticking = tpTimer.show;";
			tpWindow.root.subviews[i].onMouseExit = "tpTimer.ticking = !tpTimer.show;";
			tpWindow.root.subviews[i].onMouseDown = "true;"; // Workaround which is used to prevent the tooltip from moving even in "Head Up Display" mode
		}
	}
	
	tooltipExists = true;
}
	
// Build and show the tooltip next to the hovered gauge according to the data  obtained from its source
function ShowTooltip(animate, rearrange)
{
	if(debug > 1) PrintFunction("ShowTooltip");
	
	try
	{
		tooltipGauge = selectedGauge;
	
		if(rearrange)
		{
			if(tpWindow.fade) tpWindow.fade.kill();
			if(tpWindow.slide) tpWindow.slide.kill();
			DestroyTooltipRows();
		}
	
		// Set the timer to "Hide Gauge" mode
		tpTimer.show = false;
		tpTimer.interval = 0.1;

		var gauge = selectedGauge;

		// Get the tooltip's content from the gauge's source
		var content = gauge.sourceObj.GetTooltipContent(gauge);
		
		// Set the caption
		tpCaption.data = content.caption;
                tpCaption.size = Math.max(9, Math.min(13, 13/(tpCaption.data.length/20))); // Use a proportional text size
		tpCaption.width = -1; // Reset caption width
                  
		var valuesHoffset = 0;
		
		// Create items column
		for(var i = 0; i < content.rows.length; i++)
		{
			// Create row item
			tpRowsItems[i] = new Text(tpWindow);
			tpRowsItems[i].data = content.rows[i].item;
			tpRowsItems[i].size = 12;
			tpRowsItems[i].color = "#FFFFFF";
			tpRowsItems[i].shadow = tpTextShadow;
			tpRowsItems[i].style = "bold";
			tpRowsItems[i].hOffset = iHoffset + 10;
			tpRowsItems[i].vOffset = (i+1)*13 + 26;
			
			// Get the hOffset of the values column according to the widest item
			valuesHoffset = Math.max(valuesHoffset, iHoffset + 14 + tpRowsItems[i].width);	
		}
		
		var widestValue = 0;
		
		// Create values column
		for(var i = 0; i < content.rows.length; i++)
		{
			// Create row value
			tpRowsValues[i] = new Text(tpWindow);
			tpRowsValues[i].data = content.rows[i].value;
			tpRowsValues[i].size = 12;
			tpRowsValues[i].color = "#FFFFFF";
			tpRowsValues[i].shadow = tpTextShadow;
			tpRowsValues[i].hOffset = valuesHoffset;
			tpRowsValues[i].vOffset = (i+1)*13 + 26;
			tpRowsValues[i].truncation = "end";
			
			// Get the widest value
			widestValue = Math.max(widestValue, tpRowsValues[i].width);
		}
		
		// Set values' maximum width
		for(var i = 0; i < content.rows.length; i++)
		{
			tpRowsValues[i].width = widestValue + 9;
		}
		
		// Calculate the tooltip width - make sure all data is shown and try to show the whole caption unless it's too long
		var tooltipWidth = Math.max(Math.min(200, tpCaption.width + 17), valuesHoffset + widestValue)  + 12 - iHoffset;
                
		// Make sure the caption isn't too long
		tpCaption.width = tooltipWidth - 18;
		
		// Stretch tooltip's background to fit its content
		tpTop.width = tpBottom.width = tpCenter.width = tooltipWidth - tpLeft.width - tpRight.width;
		tpLeft.height = tpCenter.height = tpRight.height = content.rows.length*13 + 20 - tpTop.height;
		
		tpTopRight.hOffset = tpBottomRight.hOffset = tpRight.hOffset = tpTop.hOffset + tpTop.width;
		tpTopRight.hOffset = tpTop.hOffset + tpTop.width;
		
		tpBottomLeft.vOffset = tpBottom.vOffset = tpBottomRight.vOffset = tpLeft.vOffset + tpLeft.height;
		
		// Set the separator's width
		tpSeparator.width = tpRight.hOffset + tpRight.width - iHoffset - 20;
		
		// Set the tooltip window's dimensions
		tpWindow.width = tpRight.hOffset + tpRight.width + tpArrow.width - 2;
		tpWindow.height = tpBottom.vOffset + tpBottom.height;
                
		SetTooltipPosition(gauge, true);
		
		// Update hovered gauge to make sure it shows a similar level value as shown in the tooltip
		selectedGauge.Update();
		
		tpPositionTimer.ticking = true;

		if(animate) AnimateTooltip(true);
		else
		{
			tpWindow.visible = true;
			tpWindow.opacity = TOOLTIP_OPACITY;
		}
	}
	catch(e)
	{
		PrintError("ShowTooltip()", "An error occurred while the tooltip was initialized.", e);
	}
}

// Update tooltip's content
function UpdateTooltip(gauge)
{
	try
	{
		if(gauge == tooltipGauge)
		{
			if(debug > 1) PrintFunction("UpdateTooltip");
			
			var gauge = selectedGauge;

			// Get rows's information from the hovered gauge's source
			var rows = gauge.sourceObj.GetTooltipContent(gauge).rows;
			
			// Update items and values
			for(var i = 0; i < rows.length; i++)
			{
				tpRowsItems[i].data = rows[i].item;
				tpRowsValues[i].data = rows[i].value;
			}
                }
                else HideTooltip();
	}
	catch(e)
	{
			PrintError("UpdateTooltip()", "An error occurred while the tooltip was updated.", e);
	}
}

// Set tooltip window's position
function SetTooltipPosition(gauge, force)
{
	if(debug > 1) PrintFunction("SetTooltipPosition");
	
	// Continue either if forced or if the hovered gauge's position has changed
	if(force || (gauge.window.hOffset != selectedGaugeHoffset || gauge.window.vOffset != selectedGaugeVoffset))
	{
		// Update hovered gauge's position data
		selectedGaugeHoffset = gauge.window.hOffset;
		selectedGaugeVoffset = gauge.window.vOffset;
		
		// Set the arrow's and window's default positions (tooltip appears to the left of the hovered gauge)
		tpArrow.rotation = 0;
		tpArrow.hOffset = tpRight.hOffset + tpRight.width - 2;
		tpArrow.vOffset = (tpBottom.vOffset + tpBottom.height)/2 - tpArrow.height/2;
		
		// Place the tooltip next to the hovered gauge and make sure it doesn't appear off-screen:

		// First set the tooltip left to the gauge and make sure the top and bottom screen sides don't hide it
		tpWindow.hOffset = gauge.window.hOffset + gauge.background.hOffset - gauge.background.width/2 - tpWindow.width;
		tpWindow.vOffset = Math.max(screen.availTop, Math.min(screen.availHeight - tpWindow.height, gauge.window.vOffset + GAUGE_WINDOW_SIZE/2 - tpWindow.height/2));
                
		tooltipSide = "left";
		
		// If there's not enough screen space for the tooltip, place it right to the gauge
		if(tpWindow.hOffset < screen.availLeft - tpArrow.width)
		{
			tooltipSide = "right";
			tpWindow.hOffset = gauge.window.hOffset + gauge.background.hOffset + gauge.background.width/2;
				
			tpArrow.rotation = 180;
			tpArrow.hOffset = iHoffset + 2;
			tpArrow.vOffset = (tpBottom.vOffset + tpBottom.height)/2 + tpArrow.height/2;
		}
		
		tpWindow.recalcShadow();
	}	
}

// Update tooltip's skin
function UpdateTooltipSkin()
{
	if(debug) PrintFunction("UpdateTooltipSkin");

	tpArrow.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/arrow.png";
	
	// The tooltip's should always leave some space in the left side for the arrow in case it should appear in the right side of the gauge
	iHoffset = tpArrow.width - 2;

	tpTopLeft.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/topLeft.png";
	tpTopLeft.hOffset = iHoffset;

	tpTop.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/top.png";
	tpTop.hOffset = tpTopLeft.hOffset + tpTopLeft.width;

	tpTopRight.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/topRight.png";

	tpLeft.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/left.png";
	tpLeft.hOffset = tpTopLeft.hOffset;
	tpLeft.vOffset = tpTopLeft.height;

	tpCenter.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/center.png";
	tpCenter.hOffset = tpTopLeft.hOffset + tpLeft.width;
	tpCenter.vOffset = tpTop.height;

	tpRight.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/right.png";
	tpRight.vOffset = tpTopLeft.height;

	tpBottomLeft.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/bottomLeft.png";
	tpBottomLeft.hOffset = tpTopLeft.hOffset;

	tpBottom.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/bottom.png";
	tpBottom.hOffset = tpBottomLeft.hOffset + tpBottomLeft.width;

	tpBottomRight.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/bottomRight.png";

	tpSeparator.src = "Resources/Images/Tooltip Skins/" + preferences.tooltipSkin.value + "/separator.png";
	tpSeparator.hOffset = iHoffset + 10;
	tpSeparator.vOffset = 22;
}

// This function is responsible for animating the tooltip according to the tooltip animation preference
function AnimateTooltip(show)
{
	if(debug > 1) PrintFunction("AnimateTooltip");

	switch(preferences.tooltipAnimation.value)
	{
		case "None":
			if(show)
			{
				tpWindow.visible = true;
				tpWindow.opacity = TOOLTIP_OPACITY;
			}
			else
			{
				tpWindow.opacity = 0;
				tpWindow.visible = false;
			}
		break;
		case "Slide":
			if(tpWindow.fade) tpWindow.fade.kill();
			if(tpWindow.slide) tpWindow.slide.kill();
                        
			if(show)
			{
				var tooltipHoffset = tpWindow.hOffset;
				tpWindow.hOffset = tooltipSide == "left" ? tpWindow.hOffset + 30 : tpWindow.hOffset - 30;
			
				tpWindow.fade = new FadeAnimation(tpWindow, TOOLTIP_OPACITY, 300, animator.kEaseOut);
				tpWindow.slide = new MoveAnimation(tpWindow, tooltipHoffset, tpWindow.vOffset, 300, animator.kEaseOut);
				
				tpWindow.visible = true;
				animator.start(new Array(tpWindow.fade, tpWindow.slide));
			}
			else
			{
				tpWindow.fade = new FadeAnimation(tpWindow, 0, 300, animator.kEaseOut);
				tpWindow.slide = new MoveAnimation(tpWindow, tooltipSide == "left" ? tpWindow.hOffset + 30 : tpWindow.hOffset - 30, tpWindow.vOffset, 300, animator.kEaseOut);
				
				animator.runUntilDone(new Array(tpWindow.fade, tpWindow.slide));
				tpWindow.visible = false;
			}
		break;
		default:
		    if(tpWindow.fade) tpWindow.fade.kill();
                           
			if(show)
			{
			    tpWindow.visible = true;
			    tpWindow.fade = new FadeAnimation(tpWindow, TOOLTIP_OPACITY, 300, animator.kEaseOut);
                                
                                animator.start(tpWindow.fade);
			}
			else
			{
                                tpWindow.fade = new FadeAnimation(tpWindow, 0, 300, animator.kEaseOut);
                                
                                animator.runUntilDone(tpWindow.fade);
                                tpWindow.visible = false;
			}
		break;
	}
	
        if(system.platform == "macintosh")
        {
                // Make sure selected gauge os focused so it will send mouse events
                selectedGauge.window.focus(); 
        }
}

// Hide tooltip and destroy its rows
function HideTooltip()
{
	if(debug > 1) PrintFunction("HideTooltip");
        
	// Does the tooltip exist and visible?
	if(tooltipExists && tpWindow.visible)
	{
		// Set the timer to "Show Gauge" mode
		tpTimer.show = true;
		tpTimer.updateInterval();
		
		tpPositionTimer.ticking = false;
		AnimateTooltip(false);

		DestroyTooltipRows();
                
                // Make sure all gauges are unfocused
		dummyWindow.focus();
	}
}

// Remove tooltip rows when they are not needed
function DestroyTooltipRows()
{
	try
	{
		for(var i = tpRowsItems.length - 1; i >= 0; i--)
		{
			tpRowsItems[i].removeFromSuperview();
			tpRowsValues[i].removeFromSuperview();
			
			tpRowsItems[i] = tpRowsValues[i] = null;
			
			tpRowsItems.splice(i, 1);
			tpRowsValues.splice(i, 1);
		}
	}
	catch(e)
	{
		PrintError("DestroyTooltipRows()", "An error occurred while attempting to clear tooltip's content.", e);
	}
}

// Remove tooltip content to free up memory
function DestroyTooltip()
{	
	if(tooltipExists)
	{
		if(debug) PrintFunction("DestroyTooltip");
	
		tooltipExists = false;
	
		// Destory tooltip's content
		
		for(var i = tpWindow.root.subviews.length - 1 ; i >= 0; i--)
		{
			tpWindow.root.subviews[i].removeFromSuperview();
		}
		
		tpArrow = tpSeparator = tpTopLeft = tpTop = tpTopRight = tpLeft = tpCenter = tpRight = tpBottomLeft = tpBottom = tpBottomRight = tpTextShadow = tpCaption = tpPositionTimer = null;
	}
}